# -*- coding: utf-8 -*-
"""
@Time    : 2021/11/18 10:27
@Author  : lrabbit
@FileName: config_helper.py
@Software: PyCharm
@Blog    : https://www.lrabbit.life
"""

from configparser import ConfigParser
import os


class MysqlConfigClass:
    MYSQL_USER = None
    MYSQL_PASSWORD = None
    MYSQL_DATABASE = None
    MYSQL_HOST = None
    MYSQL_PORT = 3306


class RedisConfigClass:
    REDIS_PASSWORD = None
    REDIS_DATABASE = None
    REDIS_HOST = None
    REDIS_PORT = 6379


def get_config(config_path=None, env='test'):
    """

     :param config_path:
     :return:
     """
    config = ConfigParser()
    if not config_path:
        pwd = os.path.dirname(__file__)
        config_path = os.path.join(pwd, 'crawl.ini')
    config.read(config_path)
    if os.getenv("ENV") == 'server':
        env = os.getenv("ENV")
    config = config[env]
    return config


def get_mysql_config(config_path, env='test') -> MysqlConfigClass:
    if not config_path:
        raise Exception("无效的文件路径")
    config = get_config(config_path, env)
    mysqlconfig = MysqlConfigClass()
    for k, v in config.items():
        setattr(mysqlconfig, k.upper(), v)
    return mysqlconfig


def get_redis_config(config_path, env='test') -> RedisConfigClass:
    if not config_path:
        raise Exception("无效的文件路径")
    config = get_config(config_path, env)
    redisconfig = RedisConfigClass()
    for k, v in config.items():
        setattr(redisconfig, k.upper(), v)
    return redisconfig


def get_config_path(config_path_env=None):
    if not config_path_env:
        config_path_env = "config_path"
    config_path = os.environ.get(config_path_env)
    if not config_path:
        raise Exception(f"请设置环境变量{config_path_env}为ini配置文件的绝对路径")
    return config_path



import frida
import os
from threading import Thread


def start_frida_server():
    import os
    print("restart frida server")
    servername = "lrabbitarm64"
    port = "2333"
    server_path = input(f"please input frida_server path enter:/data/local/tmp/{servername}")
    if len(server_path) <= 3:
        server_path = f'/data/local/tmp/{servername}'
    killserver = "adb shell su -c killall -9 " + servername
    os.system(killserver)
    adbserver = "adb shell su -c " + server_path + " -l " + "0.0.0.0:" + port
    res = os.popen(adbserver)
    for i in res:
        print(i)


def sslbypass():
    phohe_ip = "192.168.1.97"
    t = Thread(target=start_frida_server, args=())
    t.start()
    cmd = f"frida-ps -H {phohe_ip}:2333 |grep -v xiaomi |grep -v tencent |grep -v miui"
    applciations = os.popen(cmd)
    app_dict = {}
    count = 0
    for i in applciations:
        count += 1
        app_dict[str(count)] = i.split()[-1]
    for k, v in app_dict.items():
        print(f'{k} : {v}')
    app_name = input("请输入对应的数字 eg 1 2\n")
    print(f'当前手机IP: {phohe_ip}')
    device = frida.get_device_manager().add_remote_device(f'{phohe_ip}:2333')
    packagename = app_dict[app_name]
    pid = device.spawn([packagename])
    device.resume(pid)
    device.attach(pid)


if __name__ == '__main__':
    sslbypass()



# [server]
# db_user = root
# db_password = password
# db_database = database_name
# db_host = 127.0.0.1/ you best not add into git
# workers_num = 10
# [test]
# db_user = root
# db_password = password
# db_database = database_name
# db_host = 127.0.0.1
# workers_num = 10



# -*- coding: utf-8 -*-
"""
@Time    : 2021/11/22 13:41
@Author  : lrabbit
@FileName: spider.py
@Software: PyCharm
@Blog    : https://www.lrabbit.life
"""


class Excepiton403(Exception):
    def __init__(self):
        self.__name = "exception403"


class Exception404(Exception):
    pass


class Exception500(Exception):
    pass


class ExceptionFileFieldNameError(Exception):
    pass


if __name__ == '__main__':
    print(type(Excepiton403()).__name__)



# -*- coding: utf-8 -*-
"""
@Time    : 2021/11/18 10:14
@Author  : lrabbit
@FileName: redis_helper.py
@Software: PyCharm
@Blog    : https://www.lrabbit.life
"""
import redis
from lrabbit_scrapy.common_utils.config_helper import get_redis_config, get_config_path


class RedisClient:

    def __init__(self, db=0, config_path_env=None, env='test'):
        config_path = get_config_path(config_path_env)
        redis_config = get_redis_config(config_path, env)
        self.redis_executor = redis.StrictRedis(host=redis_config.REDIS_HOST, port=redis_config.REDIS_PORT,
                                                password=redis_config.REDIS_PASSWORD,
                                                db=db,decode_responses=True)


if __name__ == '__main__':
    redis_client = RedisClient()



# -*- coding: utf-8 -*-
"""
@Time    : 2021/11/18 10:22
@Author  : lrabbit
@FileName: all_in_one.py
@Software: PyCharm
@Blog    : https://www.lrabbit.life
"""
import os
import datetime
import csv
from lrabbit_scrapy.all_excepiton import ExceptionFileFieldNameError


def get_time_format_now(option=1):
    if option == 1:
        return datetime.datetime.now().strftime("%Y-%m-%d")
    else:
        return datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S")


class FileStore:

    def __init__(self, file_path: str, filed_name: list):
        self.file_path = file_path
        self.file_name = os.path.splitext(file_path)
        self.filed_name = filed_name

    def write(self, d: dict):
        """
        :param d: dict type
        :return:
        """
        if list(d.keys()) != self.filed_name:
            raise ExceptionFileFieldNameError()

        with open(self.file_path, 'a', encoding='utf8', newline='') as f:
            dict_write = csv.DictWriter(f, fieldnames=self.filed_name)
            dict_write.writerow(d)

    def write_many(self, rows: [dict]):
        if list(d.keys()) != self.filed_name:
            raise ExceptionFileFieldNameError()
        with open(self.file_path, 'a', encoding='utf8', newline='') as f:
            dict_write = csv.DictWriter(f, fieldnames=self.filed_name)
            dict_write.writerows(rows)


if __name__ == '__main__':
    blog_file = FileStore(r"D:\PythonWorkSpace\lrabbit_scrapy\test\blogPost.csv", ["title"])
    d = {"title": "1"}
    blog_file.write(d)



import time
import traceback
import pymysql.cursors
import pymysql
import queue
import threading
import logging
from lrabbit_scrapy.common_utils.config_helper import get_mysql_config, get_config_path


class MysqlClient(object):

    def __init__(self, database=None, config_path_env=None, env='test'):
        config_path = get_config_path(config_path_env)
        mysql_config = get_mysql_config(config_path, env)
        host = mysql_config.MYSQL_HOST
        if not database:
            database = mysql_config.MYSQL_DATABASE
        user = mysql_config.MYSQL_USER
        password = mysql_config.MYSQL_PASSWORD
        port = mysql_config.MYSQL_PORT
        max_idle_time = 7 * 3600
        connect_timeout = 10
        time_zone = "+0:00"
        charset = "utf8mb4"
        sql_mode = "TRADITIONAL"
        self.host = host
        self.database = database
        self.max_idle_time = float(max_idle_time)
        args = dict(use_unicode=True, charset=charset,
                    database=database,
                    init_command=('SET time_zone = "%s"' % time_zone),
                    cursorclass=pymysql.cursors.DictCursor,
                    connect_timeout=connect_timeout, sql_mode=sql_mode)
        if user is not None:
            args["user"] = user
        if password is not None:
            args["passwd"] = password
        # We accept a path to a MySQL socket file or a host(:port) string
        if "/" in host:
            args["unix_socket"] = host
        else:
            self.socket = None
            pair = host.split(":")
            if len(pair) == 2:
                args["host"] = pair[0]
                args["port"] = int(pair[1])
            else:
                args["host"] = host
                args["port"] = 3306
        if port:
            args['port'] = port

        self._db = None
        self._db_args = args
        self._last_use_time = time.time()
        try:
            self.reconnect()
        except Exception:
            logging.error("Cannot connect to MySQL on %s", self.host,
                          exc_info=True)

    def _ensure_connected(self):
        if (self._db is None or
                (time.time() - self._last_use_time > self.max_idle_time)):
            self.reconnect()
        self._last_use_time = time.time()

    def _cursor(self):
        self._ensure_connected()
        return self._db.cursor()

    def __del__(self):
        self.close()

    def close(self):
        """Closes this database connection."""
        if getattr(self, "_db", None) is not None:
            self._db.close()
            self._db = None

    def reconnect(self):
        """Closes the existing database connection and re-opens it."""
        self.close()
        self._db = pymysql.connect(**self._db_args)
        self._db.autocommit(True)

    def query(self, query, *parameters, **kwparameters):
        """Returns a row list for the given query and parameters."""
        cursor = self._cursor()
        try:
            cursor.execute(query, kwparameters or parameters)
            result = cursor.fetchall()
            return result
        finally:
            cursor.close()

    def get(self, query, *parameters, **kwparameters):
        """Returns the (singular) row returned by the given query.
        """
        cursor = self._cursor()
        try:
            cursor.execute(query, kwparameters or parameters)
            return cursor.fetchone()
        finally:
            cursor.close()

    def execute(self, query, *parameters, **kwparameters):
        """Executes the given query, returning the lastrowid from the query."""
        cursor = self._cursor()
        try:
            cursor.execute(query, kwparameters or parameters)
            return cursor.lastrowid
        except Exception as e:
            if e.args[0] == 1062:
                pass
            else:
                traceback.print_exc()
                raise e
        finally:
            cursor.close()

    insert = execute

    ## =============== high level method for table ===================

    def table_has(self, table_name, field, value):
        if isinstance(value, str):
            value = value.encode('utf8')
        sql = 'SELECT %s FROM %s WHERE %s="%s"' % (
            field,
            table_name,
            field,
            value)
        d = self.get(sql)
        return d

    def table_insert(self, table_name, item):
        '''item is a dict : key is mysql table field'''
        fields = list(item.keys())
        values = list(item.values())
        fieldstr = ','.join(fields)
        valstr = ','.join(['%s'] * len(item))
        for i in range(len(values)):
            if isinstance(values[i], str):
                values[i] = values[i].encode('utf8')
        sql = 'INSERT INTO %s (%s) VALUES(%s)' % (table_name, fieldstr, valstr)
        try:
            last_id = self.execute(sql, *values)
            return last_id
        except Exception as e:
            if e.args[0] == 1062:
                # just skip duplicated item
                pass
            else:
                traceback.print_exc()
                print('sql:', sql)
                print('item:')
                for i in range(len(fields)):
                    vs = str(values[i])
                    if len(vs) > 300:
                        print(fields[i], ' : ', len(vs), type(values[i]))
                    else:
                        print(fields[i], ' : ', vs, type(values[i]))
                raise e

    def table_update(self, table_name, updates,
                     field_where, value_where):
        '''updates is a dict of {field_update:value_update}'''
        upsets = []
        values = []
        for k, v in updates.items():
            s = '%s=%%s' % k
            upsets.append(s)
            values.append(v)
        upsets = ','.join(upsets)
        sql = 'UPDATE %s SET %s WHERE %s="%s"' % (
            table_name,
            upsets,
            field_where, value_where,
        )
        self.execute(sql, *(values))


logger = logging.Logger(name="mysql connect")


class Connection(pymysql.connections.Connection):
    _pool = None
    _reusable_expection = (pymysql.err.ProgrammingError, pymysql.err.IntegrityError, pymysql.err.NotSupportedError)

    def __init__(self, *args, **kwargs):
        pymysql.connections.Connection.__init__(self, *args, **kwargs)
        self.args = args
        self.kwargs = kwargs

    def __exit__(self, exc, value, traceback):

        pymysql.connections.Connection.__exit__(self, exc, value, traceback)
        if self._pool:
            if not exc or exc in self._reusable_expection:
                '''reusable connection'''
                self._pool.put_connection(self)
            else:
                '''no reusable connection, close it and create a new one then put it to the pool'''
                self._pool.put_connection(self._recreate(*self.args, **self.kwargs))
                self._pool = None
                try:
                    self.close()
                    logger.warning("Close not reusable connection from pool(%s) caused by %s", self._pool.name, value)
                except Exception:
                    pass

    def _recreate(self, *args, **kwargs):
        conn = Connection(*args, **kwargs)
        logger.debug('Create new connection due to pool(%s) lacking', self._pool.name)
        return conn

    def close(self):

        if self._pool:
            self._pool.put_connection(self)
        else:
            pymysql.connections.Connection.close(self)

    def execute_query(self, query, args=(), dictcursor=False, return_one=False, exec_many=False):

        with self:
            cur = self.cursor() if not dictcursor else self.cursor(pymysql.cursors.DictCursor)
            try:
                if exec_many:
                    cur.executemany(query, args)
                else:
                    cur.execute(query, args)
            except Exception:
                raise
            # if no record match the query, return () if return_one==False, else return None
            return cur.fetchone() if return_one else cur.fetchall()


class ConnectionPool:
    _HARD_LIMIT = 200
    _THREAD_LOCAL = threading.local()
    _THREAD_LOCAL.retry_counter = 0  # a counter used for debug get_connection() method

    def __init__(self, size=10, name=None, *args, **kwargs):
        self._pool = queue.Queue(self._HARD_LIMIT)
        self._size = size if 0 < size < self._HARD_LIMIT else self._HARD_LIMIT
        self.name = name if name else '-'.join(
            [kwargs.get('host', 'localhost'), str(kwargs.get('port', 3306)),
             kwargs.get('user', ''), kwargs.get('database', '')])
        for _ in range(self._size):
            conn = Connection(*args, **kwargs)
            conn._pool = self
            self._pool.put(conn)

    def get_connection(self, timeout=1, retry_num=1) -> Connection:
        """
        timeout: timeout of get a connection from pool, should be a int(0 means return or raise immediately)
        retry_num: how many times will retry to get a connection
        """
        try:
            conn = self._pool.get(timeout=timeout) if timeout > 0 else self._pool.get_nowait()
            logger.debug('Get connection from pool(%s)', self.name)
            return conn
        except queue.Empty:
            if not hasattr(self._THREAD_LOCAL, 'retry_counter'):
                self._THREAD_LOCAL.retry_counter = 0
            if retry_num > 0:
                self._THREAD_LOCAL.retry_counter += 1
                logger.debug('Retry get connection from pool(%s), the %d times', self.name,
                             self._THREAD_LOCAL.retry_counter)
                retry_num -= 1
                return self.get_connection(timeout, retry_num)
            else:
                total_times = self._THREAD_LOCAL.retry_counter + 1
                self._THREAD_LOCAL.retry_counter = 0
                raise GetConnectionFromPoolError("can't get connection from pool({}) within {}*{} second(s)".format(
                    self.name, timeout, total_times))

    def put_connection(self, conn):
        if not conn._pool:
            conn._pool = self
        conn.cursor().close()
        try:
            self._pool.put_nowait(conn)
            logger.debug("Put connection back to pool(%s)", self.name)
        except queue.Full:
            logger.warning("Put connection to pool(%s) error, pool is full, size:%d", self.name, self.size())

    def size(self):
        return self._pool.qsize()


class GetConnectionFromPoolError(Exception):
    """Exception related can't get connection from pool within timeout seconds."""


class MysqlConnectionPool:

    def __init__(self, database=None, config_path_env=None, env='test'):
        config_path = get_config_path(config_path_env)
        mysql_config = get_mysql_config(config_path, env)
        host = mysql_config.MYSQL_HOST
        if not database:
            database = mysql_config.MYSQL_DATABASE
        user = mysql_config.MYSQL_USER
        password = mysql_config.MYSQL_PASSWORD
        port = mysql_config.MYSQL_PORT
        max_idle_time = 7 * 3600
        connect_timeout = 10
        time_zone = "+0:00"
        charset = "utf8mb4"
        sql_mode = "TRADITIONAL"
        self.host = host
        self.database = database
        self.max_idle_time = float(max_idle_time)
        args = dict(use_unicode=True, charset=charset,
                    database=database,
                    init_command=('SET time_zone = "%s"' % time_zone),
                    cursorclass=pymysql.cursors.DictCursor,
                    connect_timeout=connect_timeout, sql_mode=sql_mode)
        if user is not None:
            args["user"] = user
        if password is not None:
            args["passwd"] = password
        # We accept a path to a MySQL socket file or a host(:port) string
        if "/" in host:
            args["unix_socket"] = host
        else:
            self.socket = None
            pair = host.split(":")
            if len(pair) == 2:
                args["host"] = pair[0]
                args["port"] = int(pair[1])
            else:
                args["host"] = host
                args["port"] = 3306
        if port:
            args['port'] = port
        self._args = args
        self.pool = ConnectionPool(size=10, **self._args)

    def execute_query(self, sql):
        conn = self.pool.get_connection()
        with conn:
            res = conn.execute_query(sql)
            return res


if __name__ == '__main__':
    pass



import asyncio.queues
import asyncio
import time
import datetime
import sys
import sqlalchemy as sa
from sqlalchemy import MetaData, Table, Column, Integer
from aiomysql.sa import create_engine as aio_create_engine
from sqlalchemy import create_engine
from configparser import ConfigParser
from typing import Dict
import inspect
import csv
import os
import traceback

pwd_dir = os.path.abspath(os.getcwd())
metadata = MetaData()

config = None
last_exc = None


class TermColor:
    ATTRIBUTES = dict(
        list(zip([
            'bold',
            'dark',
            '',
            'underline',
            'blink',
            '',
            'reverse',
            'concealed'
        ],
            list(range(1, 9))
        ))
    )
    del ATTRIBUTES['']

    HIGHLIGHTS = dict(
        list(zip([
            'on_grey',
            'on_red',
            'on_green',
            'on_yellow',
            'on_blue',
            'on_magenta',
            'on_cyan',
            'on_white'
        ],
            list(range(40, 48))
        ))
    )

    COLORS = dict(
        list(zip([
            'grey',
            'red',
            'green',
            'yellow',
            'blue',
            'magenta',
            'cyan',
            'white',
        ],
            list(range(30, 38))
        ))
    )

    RESET = '\033[0m'

    @staticmethod
    def colored(text, color=None, on_color=None, attrs=None):

        if os.getenv('ANSI_COLORS_DISABLED') is None:
            fmt_str = '\033[%dm%s'
            if color is not None:
                text = fmt_str % (TermColor.COLORS[color], text)

            if on_color is not None:
                text = fmt_str % (TermColor.HIGHLIGHTS[on_color], text)

            if attrs is not None:
                for attr in attrs:
                    text = fmt_str % (TermColor.ATTRIBUTES[attr], text)

            text += TermColor.RESET
        return text


class CommonUtils:

    def __init__(self):
        pass

    @staticmethod
    def fix_str_args(args):
        return list(map(lambda x: str(x).strip(), args))

    @staticmethod
    def get_format_time(for_mat='%Y-%m-%d %H:%M:%S'):
        return TermColor.colored(datetime.datetime.now().strftime(for_mat), 'yellow').encode('utf8')

    @staticmethod
    def space_join_line_arg(*args):
        return ' '.join(args) + '\n'


class LogUtils:

    def __init__(self):
        pass

    @staticmethod
    def log_now_time_str():
        sys.stdout.buffer.write(CommonUtils.get_format_time())

    @staticmethod
    def log_str(color_str, args):
        args = CommonUtils.fix_str_args(args)
        text = ' '.join(args)
        text = color_str + ' ' + text + '\n'
        sys.stdout.buffer.write(text.encode('utf8'))

    @staticmethod
    def log_info(*args):
        color_str = TermColor.colored('[*INFO*]', 'cyan')
        LogUtils.log_str(color_str, args)

    @staticmethod
    def log_running(*args):
        color_str = TermColor.colored('[*RUNNING*]', 'yellow')
        LogUtils.log_str(color_str, args)

    @staticmethod
    def log_finish(*args):
        color_str = TermColor.colored('*FINISH*', 'green')
        LogUtils.log_str(color_str, args)

    @staticmethod
    def log_error(*args):
        color_str = TermColor.colored('[*ERROR*]', 'red')
        LogUtils.log_str(color_str, args)

    @staticmethod
    def log_to_file(file_path, line):
        """

        :param file_path: log file path
        :param line: a str type
        :return:
        """
        with open(file_path, 'a', encoding='utf8') as f:
            line = CommonUtils.space_join_line_arg(LogUtils.get_format_time(), line)
            f.write(line)


class DbUtils(LogUtils):
    """
    all_tables is store all tables sqlalchemy metadata
    """
    all_tables: Dict[str, sa.Table] = {}

    def __init__(self, spider_name):
        spider_name = spider_name.split(".")[0]
        super(DbUtils, self).__init__()
        global config
        config = ConfigParser()
        config_path = os.path.join(pwd_dir, f'{spider_name}.ini')
        config.read(config_path)
        env = os.getenv('ENV', 'test')
        config = config[env]
        self.engine = None

    async def init_engine(self):
        """
        init connect pymysql,create tables by sqlalchemy
        :return:
        """
        self.engine = await aio_create_engine(
            user=config['db_user'],
            password=config['db_password'],
            port=3306,
            host=config['db_host'],
            db=config['db_database'],
            autocommit=True
        )
        engine = create_engine(
            f'mysql+pymysql://{config["db_user"]}:{config["db_password"]}@{config["db_host"]}/{config["db_database"]}',
            echo=True,

        )
        metadata.bind = engine
        try:
            if self.__getattribute__('is_drop_tables'):
                yes = input("请输入yes,确认删除表数据")
                if yes != 'yes':
                    exit(1)
                metadata.drop_all()
                LogUtils.log_finish('已清空表')
        except Exception as e:
            LogUtils.log_info("not found is_drop_tables")
        self._generate_tables()
        metadata.create_all(engine)

    def _generate_tables(self):
        """
        get all attrs and check name ,if start_with "table" ,get it values to generate columns.

        :return:
        """
        for k, v in inspect.getmembers(self):
            if k.startswith('table'):
                table_name = k.split('_')[-1]
                try:
                    tbl = Table(table_name, metadata, Column('id', Integer, primary_key=True), *v)
                except Exception as e:
                    pass
                self.all_tables[table_name] = tbl
                LogUtils.log_finish(table_name, '创建完成')

    @asyncio.coroutine
    def insert_one(self, sql):
        """
        :param sql: this sql you can sqlalchemy api generate a sql  or you can just write origin sql
        :return:
        """
        with (yield from self.engine) as conn:
            yield from conn.execute(sql)

    @asyncio.coroutine
    def query(self, sql):
        with (yield from self.engine) as conn:
            res = yield from conn.execute(sql)
            res = yield from res.fetchall()
            return res

    @asyncio.coroutine
    def execute(self, sql):
        with (yield from self.engine) as conn:
            yield from conn.execute(sql)


class FileStore:

    def __init__(self, file_name, headers, reset_task_list):
        self.file_name = f'{file_name}.csv'
        self.headers = headers
        self.reset_task_list = reset_task_list
        self.write_headers()

    def write_headers(self):
        """
        write headers
        :return:
        """
        if not os.path.exists(os.path.join(pwd_dir, self.file_name)) or self.reset_task_list:
            with open(os.path.join(pwd_dir, self.file_name), 'w', encoding='utf8', newline='') as f:
                dict_write = csv.DictWriter(f, fieldnames=self.headers)
                dict_write.writeheader()

    def write(self, d):
        """

        :param d: dict type
        :return:
        """
        with open(os.path.join(pwd_dir, self.file_name), 'a', encoding='utf8', newline='') as f:
            dict_write = csv.DictWriter(f, fieldnames=self.headers)
            dict_write.writerow(d)


class WriteUtil(LogUtils):
    """
     all_files : store all FileStore object,you can pass a name to write data to different files
    """
    all_files: Dict[str, FileStore] = {}

    def __init__(self):

        super(WriteUtil, self).__init__()
        self._generate_files()

    def _generate_files(self):
        try:
            reset_task_list = self.__getattribute__('reset_task_list')
        except Exception as e:
            self.log_info('not found reset_task_list option ')
            return
        for k, v in inspect.getmembers(self):
            if k.startswith('file'):
                file_name = k.split("_")[-1]
                self.all_files[file_name] = FileStore(file_name, v, reset_task_list)
                LogUtils.log_finish(f'创建{file_name}存储文件成功')


class BaseSpider(DbUtils, WriteUtil):
    """
    spider_name:default is filename
    """

    def __init__(self, spider_name: str):
        # in windows spider_name maybe is .\crawl.py ,so we need trim it
        if spider_name.startswith(".\\"):
            spider_name = spider_name.replace(".\\", "")
        spider_name = spider_name.split('.')[0]
        DbUtils.__init__(self, spider_name)
        WriteUtil.__init__(self)
        self.task_queue = asyncio.queues.Queue()
        self.task_list = []
        self.redis = None
        self.db = None
        self.spider_name = spider_name
        self.start_time = None
        self.finish_file_name = None
        self.all_file_name = None
        self.init_file_name(spider_name)
        self.config = ConfigParser()
        config_path = os.path.join(pwd_dir, f'{spider_name}.ini')
        self.config.read(config_path)
        env = os.getenv('ENV', 'test')
        self.config = self.config[env]

    def init_file_name(self, spider_name):
        """

        :param spider_name:generate task_list files
        :return:
        """
        spider_name = spider_name.split('.')[0]
        self.finish_file_name = f'{spider_name}_finish.log'
        self.all_file_name = f'{spider_name}_all.log'

    def get_tasks_list_by_file(self):
        """
        read from all.log and finish.log and get not finish task
        :return:
        """
        finish_set = set()
        all_set = set()
        if os.path.exists(os.path.join(pwd_dir, self.all_file_name)):
            with open(self.all_file_name, 'r', encoding='utf8') as f:
                for line in f.readlines():
                    all_set.add(line)
        if os.path.exists(os.path.join(pwd_dir, self.finish_file_name)):
            with open(self.finish_file_name, 'r', encoding='utf8') as f:
                for line in f.readlines():
                    finish_set.add(line)
        return list(all_set.difference(finish_set))

    async def _generate_task(self):
        """
        init task_list data
        :return:
        """
        try:
            reset_task_list = self.__getattribute__('reset_task_list')
        except Exception as e:
            LogUtils.log_info("not found reset_task_list")
            return
        if not os.path.exists(os.path.join(pwd_dir, self.all_file_name)) or reset_task_list:
            if os.path.exists(os.path.join(pwd_dir, self.all_file_name)):
                os.remove(os.path.join(pwd_dir, self.all_file_name))
            if os.path.exists(os.path.join(pwd_dir, self.finish_file_name)):
                os.remove(os.path.join(pwd_dir, self.finish_file_name))
            try:
                generate_callback = self.__getattribute__('create_tasks')
            except Exception as e:
                LogUtils.log_info("not found create_tasks")
                return
            task_lists = await generate_callback()
            task_lists = list(set(task_lists))
            for task in task_lists:
                with open(os.path.join(pwd_dir, self.all_file_name), 'a', encoding='utf8') as f:
                    f.write(str(task))
                    f.write('\n')
        else:
            task_lists = self.get_tasks_list_by_file()
        for task in task_lists:
            self.task_queue.put_nowait(str(task).strip())
        for _ in range(1):
            self.task_queue.put_nowait(None)

    async def base_worker(self):
        """
        every woker is in while, get task form task_queue
        :return:
        """
        try:
            worker_callback = self.__getattribute__('worker')
        except Exception as e:
            LogUtils.log_info("not found worker")
        while True:
            global last_exc
            task = await self.task_queue.get()
            if task:
                self.log_running(task)
                try:
                    await worker_callback(task)
                    with open(self.finish_file_name, 'a', encoding='utf8') as f:
                        f.write(str(task).strip())
                        f.write('\n')
                    self.log_finish(task)
                except Exception as e:
                    if str(traceback.format_exc()) != last_exc:
                        with open("error.log", 'a', encoding='utf8') as f:
                            f.write(datetime.datetime.now().strftime("%Y-%m-%d %H:%M:%S"))
                            f.write('\n')
                            f.write(traceback.format_exc())
                        last_exc = str(traceback.format_exc())
                self.task_queue.task_done()
            else:
                print("正在等待最后的任务完成，默认等待5秒")
                await asyncio.sleep(5)
                total_time = time.monotonic() - self.started_time
                for task in self.task_list:
                    task.cancel()
                print(f"除去等待时间，本次代码运行花费时间为: {total_time - 2}")
                exit(0)

    async def wait_all_task(self):
        """
        wait all worker is finished
        :return:
        """
        await self.task_queue.join()

    def run(self):
        """
        main function,scrapy start with it
        :return:
        """
        loop = asyncio.get_event_loop()
        is_open_mysql = self.__getattribute__("is_open_mysql")
        if is_open_mysql:
            loop.run_until_complete(self.init_engine())
        loop.run_until_complete(self._generate_task())
        self.task_list = []
        for i in range(int(self.config['workers_num'])):
            task = loop.create_task(self.base_worker())
            self.task_list.append(task)
        self.started_time = time.monotonic()
        loop.run_until_complete(self.wait_all_task())




# -*- coding: utf-8 -*-
"""
@Time    : 2021/11/18 10:14
@Author  : lrabbit
@FileName: redis_helper.py
@Software: PyCharm
@Blog    : https://www.lrabbit.life
"""

import datetime
import os


class TermColor:
    ATTRIBUTES = dict(
        list(zip([
            'bold',
            'dark',
            '',
            'underline',
            'blink',
            '',
            'reverse',
            'concealed'
        ],
            list(range(1, 9))
        ))
    )
    del ATTRIBUTES['']

    HIGHLIGHTS = dict(
        list(zip([
            'on_grey',
            'on_red',
            'on_green',
            'on_yellow',
            'on_blue',
            'on_magenta',
            'on_cyan',
            'on_white'
        ],
            list(range(40, 48))
        ))
    )

    COLORS = dict(
        list(zip([
            'grey',
            'red',
            'green',
            'yellow',
            'blue',
            'magenta',
            'cyan',
            'white',
        ],
            list(range(30, 38))
        ))
    )

    RESET = '\033[0m'

    @staticmethod
    def colored(text, color=None, on_color=None, attrs=None):

        if os.getenv('ANSI_COLORS_DISABLED') is None:
            fmt_str = '\033[%dm%s'
            if color is not None:
                text = fmt_str % (TermColor.COLORS[color], text)

            if on_color is not None:
                text = fmt_str % (TermColor.HIGHLIGHTS[on_color], text)

            if attrs is not None:
                for attr in attrs:
                    text = fmt_str % (TermColor.ATTRIBUTES[attr], text)

            text += TermColor.RESET
        return text


class CommonUtils:

    def __init__(self):
        pass

    @staticmethod
    def fix_str_args(args):
        return list(map(lambda x: str(x).strip(), args))

    @staticmethod
    def get_format_time(for_mat='%Y-%m-%d %H:%M:%S'):
        return TermColor.colored(datetime.datetime.now().strftime(for_mat), 'yellow')

    @staticmethod
    def space_join_line_arg(*args):
        return ' '.join(args) + '\n'


class LogUtils:

    def __init__(self):
        pass

    @staticmethod
    def log_now_time_str():
        # fix buffer cache
        # sys.stdout.buffer.write()
        print(CommonUtils.get_format_time())

    @staticmethod
    def log_str(color_str, args):
        args = CommonUtils.fix_str_args(args)
        text = ' '.join(args)
        text = color_str + ' ' + text + '\n'
        # fix buffer cache
        # sys.stdout.buffer.write(text.encode('utf8'))
        print(text,end='')

    @staticmethod
    def log_info(*args):
        color_str = TermColor.colored('[*INFO*]', 'cyan')
        LogUtils.log_str(color_str, args)

    @staticmethod
    def log_running(*args):
        color_str = TermColor.colored('[*RUNNING*]', 'yellow')
        LogUtils.log_str(color_str, args)

    @staticmethod
    def log_finish(*args):
        color_str = TermColor.colored('[*FINISH*]', 'green')
        LogUtils.log_str(color_str, args)

    @staticmethod
    def log_error(*args):
        color_str = TermColor.colored('[*ERROR*]', 'red')
        LogUtils.log_str(color_str, args)

    @staticmethod
    def log_to_file(file_path, line):
        """

        :param file_path: log file path
        :param line: a str type
        :return:
        """
        with open(file_path, 'a', encoding='utf8') as f:
            line = CommonUtils.space_join_line_arg(LogUtils.get_format_time(), line)
            f.write(line)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="lrabbit_scrapy",
    version="2.0.4",
    author="lrabbit",
    author_email="709343607@qq.com",
    description="this is a small spider,you can easy running. When you often need to crawl a single site, you don't have to redo some repeated code every time, using this small framework you can quickly crawl data into a file or database.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/litter-rabbit/lrabbit_scrapy",
    project_urls={
        "Bug Tracker": "https://github.com/litter-rabbit/lrabbit_scrapy/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "parsel == 1.6.0",
        "requests >= 2.26.0",
        "PyMySQL >= 0.9.3",
        "redispy >= 3.0.0"
    ],
    package_dir={"": "src"},
    packages=setuptools.find_packages(where="src"),
    python_requires=">=3.6",
)



from lrabbit_scrapy import BaseSpider
import sqlalchemy as sa


class Spider(BaseSpider):
    # setup
    is_open_mysql = False
    is_drop_tables = False
    # reset all tasks,files,this is may delete all data files
    reset_task_list = False

    """
        not call a method or attribute start_with of 'file','table'
    """
    # datastore
    table_table1 = [
        sa.Column('val', sa.String(255)),
    ]

    # file_store
    file_blogPost = [
        'id', 'title', 'datetime', 'content'
    ]

    def __init__(self, spider_name):
        super(Spider, self).__init__(spider_name)

    async def worker(self, task):
        """

        code your worker method

        :param task:
        :return:
        """
        """
         mysql work method
        """
        # await self.insert_one(self.tables['table1'].insert().values(val=str(task)))
        # res = await self.query(self.tables['table1'].select())
        # res = await res.fetchall()

        """
         want to see how to work,uncomment beyond code
        """
        url = f"http://www.lrabbit.life/post_detail/?id={task}"
        selector = await get_css_selector_by_url(url)
        title = selector.css(".detail-title h1::text").get()
        datetime = selector.css(".detail-info span::text").get()
        content = ''.join(selector.css(".detail-content *::text").getall())
        data = {"id": task, 'title': title, 'datetime': datetime, 'content': content}
        if title:
            self.all_files['blogPost'].write(data)

    async def create_tasks(self):
        return [i for i in range(100)]


if __name__ == '__main__':
    s = Spider(__file__)
    s.run()


import os
from pathlib import Path
from lrabbit_scrapy.common_utils.print_log_helper import LogUtils

base_dir = Path(__file__).resolve().parent


def newspider(spider_name):
    copy_new_name_file(spider_name, f'{spider_name}.py', 'crawl.py')
    copy_new_name_file(spider_name, f'{spider_name}.ini', 'config.py', is_config=True)
    LogUtils.log_finish('创建项目成功')


def copy_new_name_file(spider_name, new_name, src_name, is_config=False):
    dst_path = os.path.abspath(os.getcwd())
    if not os.path.exists(os.path.join(dst_path, spider_name)):
        os.mkdir(os.path.join(dst_path, spider_name))
    dst_path = os.path.join(dst_path, spider_name)
    dst_file = os.path.join(dst_path, new_name)
    src_file = os.path.join(base_dir, src_name)
    if os.path.exists(dst_file):
        raise Exception(f'please remove your file {dst_file}')
    f2 = open(dst_file, 'a')
    with open(src_file, 'r') as f:
        for line in f.readlines():
            if is_config:
                line = line.replace("# ", "")
            f2.write(line)
    f2.close()


def run(argv, opts):
    if argv != "newscrapy":
        raise Exception(f'{argv} command not found,Do you mean lrabbit_spider newscrapy')
    newspider(opts)


if __name__ == '__main__':
    import sys

    argv = sys.argv[1]
    opts = sys.argv[2]
    run(argv, opts)



# -*- coding: utf-8 -*-
"""
@Time    : 2021/11/18 10:21
@Author  : lrabbit
@FileName: network_helper.py
@Software: PyCharm
@Blog    : https://www.lrabbit.life
"""

import urllib3
import requests
from lrabbit_scrapy.all_excepiton import Excepiton403, Exception404, Exception500

urllib3.disable_warnings(urllib3.exceptions.InsecureRequestWarning)


class RequestSession:

    def __init__(self, proxies=None, timeout=15, headers=None):
        self.proxies = proxies
        self.timeout = timeout
        self.session = requests.session()
        self.session.headers = headers

    def send_request(self, method='GET', url=None, headers=None, data=None) -> requests.Response:

        if method == 'GET':
            res = self.session.get(url, proxies=self.proxies, verify=False, timeout=self.timeout)
        else:
            if isinstance(data, dict):
                res = self.session.post(url, json=data, proxies=self.proxies, headers=headers, verify=False,
                                        timeout=self.timeout)
            elif isinstance(data, str):
                res = self.session.get(url, data=data, proxies=self.proxies, headers=headers, verify=False,
                                       timeout=self.timeout)
            else:
                raise Exception("no data post")
        return self.deal_res(res)

    def download_file_by_url(self, out_file_path, url, headers=None):
        res = self.session.get(url=url, headers=headers, proxies=self.proxies, verify=False, stream=True,
                               )
        with open(out_file_path, 'wb') as f:
            for chunk in res.iter_content(1024):
                if chunk:
                    f.write(chunk)
        return out_file_path

    def deal_res(self, res):
        if res.status_code == 403:
            raise Excepiton403()
        elif res.status_code == 404:
            raise Exception404()
        elif res.status_code == 500:
            raise Exception500()
        return res


if __name__ == '__main__':
    session = RequestSession()



# -*- coding: utf-8 -*-
"""
@Time    : 2021/11/18 11:41
@Author  : lrabbit
@FileName: spider.py
@Software: PyCharm
@Blog    : https://www.lrabbit.life
"""
import traceback
from lrabbit_scrapy.common_utils.mysql_helper import MysqlClient, MysqlConnectionPool
from lrabbit_scrapy.common_utils.redis_helper import RedisClient
from threading import Thread
from lrabbit_scrapy.common_utils.all_in_one import get_time_format_now
from lrabbit_scrapy.common_utils.print_log_helper import LogUtils


class LrabbitSpider:

    def __init__(self):

        self._init_config()
        self.mysql_client_pool = MysqlConnectionPool(config_path_env=self.config_env_name)
        self.redis_client = RedisClient(config_path_env=self.config_env_name, db=self.redis_db_config)
        spider_task_key = f'list:{self.__getattribute__("spider_name")}'
        self.spider_name = self.__getattribute__("spider_name")
        self.task_list_redis_key = spider_task_key
        self.success_count_all_key = f"success:count:{self.spider_name}"
        self.success_count_day_key = f"success:count:{self.spider_name}:{get_time_format_now()}"
        self.fail_count_all_key = f"fail:count:{self.spider_name}"
        self.fail_count_day_key = f"fail:count:{self.spider_name}:{get_time_format_now()}"
        self.thread_task_list = []
        self.task_list = []

    def _init_config(self):

        default_config = {
            "thread_mysql_open": False,
            "max_thread_num": 10,
            "reset_task_config": False,
            "loop_task_config": False,
            "remove_confirm_config": False,
            "config_env_name": "config_path",
            "redis_db_config": 0,
            "debug_config": True
        }
        for k, v in default_config.items():
            try:
                self.__getattribute__(k)
            except Exception as e:
                self.__setattr__(k, v)

    def _send_task_redis(self, task_list):
        for task in task_list:
            LogUtils.log_info("new task", task)
            self.redis_client.redis_executor.sadd(self.task_list_redis_key, task)

    def update_stat_redis(self):
        """
         success:count_all   success:count:spider_name
         success:count:day   success:count:spider_name:2021-11-11
        :return:
        """
        day = get_time_format_now()
        self.success_count_day_key = f"success:count:{self.spider_name}:{day}"
        self.redis_client.redis_executor.incr(self.success_count_all_key)
        self.redis_client.redis_executor.incr(self.success_count_day_key)

    def _init_task_list(self):

        if self.reset_task_config or not self.redis_client.redis_executor.exists(self.task_list_redis_key):

            LogUtils.log_info("init task list")
            generate_task_list_callback = self.__getattribute__("init_task_list")
            if self.redis_client.redis_executor.exists(self.task_list_redis_key):

                LogUtils.log_info("already exists", self.task_list_redis_key, "task list", "count",
                                  self.redis_client.redis_executor.scard(self.task_list_redis_key))
                try:
                    remove_confirm_config = self.__getattribute__("remove_confirm_config")
                    if not remove_confirm_config:
                        option = input("please input y to delete task list and add new task")
                        if option != 'y':
                            exit(-1)
                except AttributeError as e:
                    option = input("please input y to delete task list and add new task")
                    if option != 'y':
                        exit(-1)
                except Exception as e:
                    pass
                self.redis_client.redis_executor.delete(self.task_list_redis_key)

            generate_task_all = generate_task_list_callback()
            count = self.redis_client.redis_executor.scard(self.task_list_redis_key)
            if count >= 1:
                LogUtils.log_info("already init task")
                return
            if len(generate_task_all) < 10:
                for item in generate_task_all:
                    LogUtils.log_info("new task", item)
                    self.redis_client.redis_executor.sadd(self.task_list_redis_key, item)
            else:
                thread_num = 10
                step = len(generate_task_all) // thread_num
                send_thread_list = []
                for i in range(thread_num):
                    if i == thread_num - 1:
                        t = Thread(target=self._send_task_redis, args=(generate_task_all[i * step:],))
                    else:
                        t = Thread(target=self._send_task_redis, args=(generate_task_all[(i * step):(i + 1) * step],))
                    t.start()
                    send_thread_list.append(t)
                for t in send_thread_list:
                    t.join()
            LogUtils.log_finish("init task list success")

        task_count = self.redis_client.redis_executor.scard(self.task_list_redis_key)
        LogUtils.log_info("current task count", task_count)
        try:
            remove_confirm_config = self.__getattribute__("remove_confirm_config")
            if not remove_confirm_config:
                option = input("please input y to continue")
                if option != 'y':
                    exit(-1)
        except AttributeError as e:
            option = input("please input y to continue")
            if option != 'y':
                exit(-1)
        except Exception as e:
            pass

    def _run(self):
        self._init_task_list()
        try:
            worker_callback = self.__getattribute__("worker")
        except Exception as e:
            LogUtils.log_error("you not define worker function")
            exit(-1)

        def self_loop_call_back():
            while True:
                task = self.redis_client.redis_executor.spop(self.task_list_redis_key)
                if not task:
                    break
                try:
                    self.task_list.append(task)
                    if self.thread_mysql_open:
                        _mysql_client = MysqlClient(config_path_env=self.config_env_name)
                        worker_callback(task, _mysql_client)
                    else:
                        worker_callback(task)
                except Exception as e:
                    name_exception = type(e).__name__.lower()
                    if self.debug_config:
                        traceback.print_exc()
                    else:
                        LogUtils.log_error(task, name_exception, e.__getattribute__('args'))
                    self.redis_client.redis_executor.sadd(
                        f"list:error:count:{self.spider_name}:{name_exception}:{get_time_format_now()}",
                        task)
                    self.fail_count_day_key = f"fail:count:{self.spider_name}:{get_time_format_now()}"
                    self.redis_client.redis_executor.incr(self.fail_count_day_key)
                    self.redis_client.redis_executor.incr(self.fail_count_all_key)
                try:
                    self.task_list.remove(task)
                except Exception as e:
                    pass

        self.task_list = []
        self.process_list = []

        for _ in range(self.max_thread_num):
            t = Thread(target=self_loop_call_back, args=())
            t.start()
            self.process_list.append(t)
        for t in self.process_list:
            t.join()

    def _menu(self):
        import sys
        options = sys.argv[1:]
        if len(options) > 0:
            if options[0] == 'stat':
                LogUtils.log_info("remain task list", self.redis_client.redis_executor.scard(self.task_list_redis_key))
                print("\n")
                day = get_time_format_now()
                self.success_count_day_key = f"success:count:{self.spider_name}:{day}"
                LogUtils.log_finish("today success count",
                                    self.redis_client.redis_executor.get(self.success_count_day_key))
                LogUtils.log_error("today fail count", self.redis_client.redis_executor.get(self.fail_count_day_key))
                LogUtils.log_finish("all success count",
                                    self.redis_client.redis_executor.get(self.success_count_all_key))
                LogUtils.log_error("all fail count", self.redis_client.redis_executor.get(self.fail_count_all_key))
                print("\n")
                LogUtils.log_error("404 status_code count",
                                   self.redis_client.redis_executor.scard(
                                       f"list:error:count:{self.spider_name}:exception404:{get_time_format_now()}"))
                LogUtils.log_error("403 status_code count",
                                   self.redis_client.redis_executor.scard(
                                       f"list:error:count:{self.spider_name}:exception403:{get_time_format_now()}"))
                LogUtils.log_error("500 status_code count",
                                   self.redis_client.redis_executor.scard(
                                       f"list:error:count:{self.spider_name}:exception500:{get_time_format_now()}"))
            else:
                LogUtils.log_error(" you can add stat option ,check scrapy stat")
            exit(-1)

    def run(self):
        while True:
            self._menu()
            try:
                self._run()
                if self.loop_task_config:
                    continue
                break
            except KeyboardInterrupt as e:
                # when you keyboard break,need give this task back
                while True:
                    if len(self.task_list) == 0:
                        break
                    task = self.task_list.pop()
                    if not task:
                        break
                    self.redis_client.redis_executor.sadd(self.task_list_redis_key, task)
        LogUtils.log_now_time_str()
        LogUtils.log_finish("抓取完毕")


if __name__ == '__main__':
    spider = LrabbitSpider()
